from odoo_library.res_partner import ResPartnerModel
from odoo_library.sale_order import SaleOrderModel

# You can also instantiate the libraries if you want to provide pre-configured instances

res_partner_model_instance = ResPartnerModel()
sale_order_model_instance = SaleOrderModel()

# Running the Flask apps by default when the package is imported

res_partner_model_instance.run()
sale_order_model_instance.run()



from flask import Flask, request, jsonify
import xmlrpc.client

class SaleOrderModel:
    def __init__(self):
        # Allow passing an external Flask app instance or create a new one
        self.app = Flask(__name__)
        self.register_routes()

    def register_routes(self):
        self.app.route('/create_sale_order', methods=['POST'])(self.create_sale_order)
        self.app.route('/get_sale_order_data', methods=['POST'])(self.get_sale_order_data)
        self.app.route('/confirm_sale_order', methods=['POST'])(self.confirm_sale_order)


# =================================================================================================================
    def connect_to_odoo(self, data):
        odoo_url = data.get('odoo_server_url')
        database = data.get('database_name')
        username = data.get('odoo_username')
        password = data.get('odoo_password')

        if not all([odoo_url, database, username, password]):
            return jsonify({'error': 'Missing Odoo XML-RPC configuration data'}), 400

        common = xmlrpc.client.ServerProxy(f'{odoo_url}/xmlrpc/2/common')
        uid = common.authenticate(database, username, password, {})
        models = xmlrpc.client.ServerProxy(f'{odoo_url}/xmlrpc/2/object')

        return models, uid, database, password
    

# =================================================================================================================
    def create_sale_order(self, data):
        models, uid, database, password = self.connect_to_odoo(data)

        try:
            if 'customerId' in data or 'customerNumber' in data:
                partnerId = data.get('customerId') or models.execute_kw(database, uid, password, 'res.partner', 'search',
                                                                    [['|', ["phone", "=", data.get('customerNumber')],
                                                                      ["mobile", "=", data.get('customerNumber')]]])
        except xmlrpc.client.Fault as e:
            partnerId = None  # Set to None if an exception occurs

        try:
            if 'resellerId' in data or 'resellerNumber' in data:
                resellerId = data.get('resellerId') or models.execute_kw(database, uid, password, 'res.partner', 'search',
                                                                         [['|', ["phone", "=", data.get('resellerNumber')],
                                                                           ["mobile", "=", data.get('resellerNumber')]]])
            else:
                resellerId = None
        except xmlrpc.client.Fault as e:
            resellerId = None  # Set to None if an exception occurs

        saleObj = {
            **({'name': str(data.get('name')) } if 'name' in data and data['name'] else {}),
            **({'partner_id': partnerId[0] } if partnerId is not None else {}),
            **({'l10n_in_reseller_partner_id': resellerId[0] } if resellerId is not None else {}),
            **({'validity_date': data.get('expirationDate')} if 'expirationDate' in data and data['expirationDate'] else {}),
            **({'date_order': data.get('quotationDate')} if 'quotationDate' in data and data['quotationDate'] else {}),
            **({'pricelist_id': (models.execute_kw(database, uid, password, 'product.pricelist', 'search',
                                                    [[["name", "=", data.get('pricelist')]]])
                                 if 'pricelist' in data else None)} if 'pricelist' in data and data['pricelist'] else {}),
            'order_line': [],
        }
        # Add customisable keys
        for key, value in data.items():
            if key.startswith('custom_'):
                suffix = key[len('custom_'):]
                saleObj[suffix] = value
        # Add order lines if order line data is provided
        productIdsArray = []
        productVariantIdsArray = []

        if 'orderLine_productNames' in data:
            for productName in data.get('orderLine_productNames'):
                if productName != '':
                    productId = models.execute_kw(database, uid, password, 'product.template', 'search_read',
                                                  [[["name", "=", productName]]])
                    productIdsArray.append(productId[0]['id'])
                    productVariantIdsArray.append(productId[0]['product_variant_id'][0])

        if 'orderLine_productId' in data:
            for productIdd in data.get('orderLine_productId'):
                if productIdd != '':
                    productId = models.execute_kw(database, uid, password, 'product.template', 'search_read',
                                                  [[["id", "=", productIdd]]])
                    productIdsArray.append(productId[0]['id'])
                    productVariantIdsArray.append(productId[0]['product_variant_id'][0])

        description = data.get('orderLine_description', [])
        quantity = data.get('orderLine_quantity', [])
        unitPrice = data.get('orderLine_unitPrice', [])
        taxes = data.get('orderLine_taxes', [])
        discount = data.get('orderLine_discount', [])

        salesObjDynamicPayload = []

        for index in range(max(len(description), len(quantity), len(unitPrice), len(taxes), len(discount),
                               len(productIdsArray), len(productVariantIdsArray))):
            product_id = productIdsArray[index] if index < len(productIdsArray) else False
            variant_id = productVariantIdsArray[index] if index < len(productVariantIdsArray) else False
            description_val = description[index] if index < len(description) else ''
            quantity_val = quantity[index] if index < len(quantity) else 0.0
            unit_price_val = unitPrice[index] if index < len(unitPrice) else 0.0
            tax_id_val = [(6, 0, [taxes[index]])] if index < len(taxes) and taxes[index] else []
            discount_val = discount[index] if index < len(discount) else 0.0

            salesObjDynamicPayload.append((0, 0, {
                'product_id': int(variant_id),
                **({'name': str(description_val) } if description_val is not None else {}),
                **({'product_uom_qty': float(quantity_val) } if quantity_val is not None else 1.0),
                **({'price_unit': float(unit_price_val) } if unit_price_val is not None else {}),
                'tax_id': tax_id_val,
                **({'discount': float(discount_val) } if discount_val is not None else 0.0)
            }))

        saleObj['order_line'] = salesObjDynamicPayload
        createSalesOrder = models.execute_kw(database, uid, password, 'sale.order', 'create', [saleObj])

        return jsonify({'sale_order_id': createSalesOrder})
    

# =================================================================================================================
    def get_sale_order_data(self, data):
        models, uid, database, password = self.connect_to_odoo(data)

        try:
            order_id = data.get('orderID')
            if not order_id:
                return jsonify({'error': 'Missing orderID in the request'}), 400

            # Fetch sales order data
            sale_order_data = models.execute_kw(database, uid, password, 'sale.order', 'search_read', [[["name", "=", order_id]]])

            if not sale_order_data:
                return jsonify({'error': f'Sale order with ID {order_id} not found'}), 404

            # You can process the sale_order_data as needed
            return jsonify({'sale_order_data': sale_order_data})

        except xmlrpc.client.Fault:
            return jsonify({'error': 'Error fetching sale order data'}), 500
        

# =================================================================================================================
    def confirm_sale_order(self, data):
        models, uid, database, password = self.connect_to_odoo(data)

        try:
            order_id = data.get('orderID')
            if not order_id:
                return jsonify({'error': 'Missing orderID in the request'}), 400
            # Confirm the Sale Order
            confirmRental = models.execute_kw(database, uid, password, 'sale.order', 'action_confirm', [[order_id]])
            response = {
                            "code": 200,
                            "status": "success",
                            "message": f"Sale Order Confirmed Successfully!",
                            "order_id": order_id  # Add the order ID here
                        }
            # You can process the sale_order_data as needed
            return jsonify(response)

        except xmlrpc.client.Fault:
            return jsonify({'error': 'Error fetching sale order data'}), 500



    def run(self, run_server=False):
        # Run the Flask app only if explicitly requested
        if run_server:
            self.app.run()



from setuptools import setup, find_packages

setup(
    name='OdooXMLRPCLibrary',
    version='0.3.10',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Flask',
        # 'xmlrpc.client',  # Corrected import for xmlrpc.client
        # Add any other dependencies here
    ],
    entry_points={
        'console_scripts': [
            #--------------- ↓ ResPartnerModel ↓ ------------------------
            'create_contact = OdooXMLRPCLibrary.ResPartnerModel:create_contact',
            'get_contact_data = OdooXMLRPCLibrary.ResPartnerModel:get_contact_data',
            #--------------- ↓ SaleOrderModel ↓ ------------------------
            'create_sale_order = OdooXMLRPCLibrary.SaleOrderModel:create_sale_order',
            'get_sale_order_data = OdooXMLRPCLibrary.SaleOrderModel:get_sale_order_data',
            'confirm_sale_order = OdooXMLRPCLibrary.SaleOrderModel:confirm_sale_order',
        ],
    },
    author='Aditya Irri',
    author_email='adityairri@gmail.com',
    description='OdooXMLRPCLibrary is a Python library that simplifies interaction with the Odoo ERP system using XML-RPC API. With modules like create_sales_order and create_contact, it provides an easy-to-use interface for managing rental orders and contacts in the Odoo platform. Whether youre integrating Odoo functionality into your application or automating business processes, OdooXMLRPCLibrary streamlines the communication process, making it efficient and developer-friendly.',
    long_description_content_type='text/markdown',
    long_description='''
    # OdooXMLRPCLibrary

OdooXMLRPCLibrary is a Python library that simplifies interaction with the Odoo ERP system using the XML-RPC API. It provides modules for Creating contacts, Creating Sales Order in the Odoo platform.

## Features

*res.partner* Model
- Add a new contact in the Odoo platform.
- Get Contact Details by Phone Number or Name.

*sale.order* Model
- Add Sales Order in the Odoo platform.
- Get Sales Order Data by Order ID.
- Confirm a Sales Order.

*More Modules and Features Coming Soon*

## Installation

To install the library, you can use pip:

```bash
pip install OdooXMLRPCLibrary
```

## How To Use?

**Importing**
```python
from odoo_library.res_partner import ResPartnerModel
from odoo_library.sale_order import SaleOrderModel
```

**Code Usage**
```python
# -------------------------- Example code to create a Contact --------------------------

# Create instances of the libraries
res_partner_model_instance = ResPartnerModel()

# Run the Instance
res_partner_model_instance.run()

# Prepare Data
data = {
    "odoo_server_url": "https://exampledb.odoo.com/",   # Your Odoo server URL here (with http or https)
    "database_name": "exampledb",                       # The database name on your Odoo Server 
    "odoo_username": "DB Username",                     # The username for your Odoo Database
    "odoo_password": "DB Password",                     # The password for your Odoo Database user
    "contact_name": "",                                 # Name of the contact
    "company_name": "",                                 # Company name of the contact
    "company_type": "company",                          # Type of company ("company" or "person")
    "address_type": "invoice",                          # Address type ("delivery" or "invoice")
    "street1": "Test Street 1",                         # First line of address
    "street2": "Test Street 2",                         # Second line of address (optional)
    "city": "Hyderabad",                                # City
    "state": "Telangana",                               # State
    "country": "India",                                 # Country
    "zip": "500032",                                    # Zip Code
    "gst_treatment": "",                                # GST Treatment if any ("registered" or "unregistered
    "vat": "",                                          # VAT number (if any)
    "job_position": "",                                 # Job Position (If applicable)
    "phone": "9988776655",                              # Phone Number
    "mobile": "",                                       # Mobile Number (Optional)
    "email": "Email@example.com",                       # Email ID
    "website": "https://example.com",                   # Website URL (Optional)
    "title": "",                                        # Title (Optional)
    "tags": ""                                          # Tags (Comma separated list, Optional)
}

# Send data to the function
response = res_partner_model_instance.create_contact(data)

#Get Response
return (response)




# -------------------------- Example code to Get Contact Data --------------------------

# Create instances of the libraries
res_partner_model_instance = ResPartnerModel()

# Run the Instance
res_partner_model_instance.run()

# Prepare Data
# Can use either 'name' or 'phone' to get the contact details 
data = {
    "odoo_server_url": "https://exampledb.odoo.com/",   # Your Odoo server URL here (with http or https)
    "database_name": "exampledb",                       # The database name on your Odoo Server 
    "odoo_username": "DB Username",                     # The username for your Odoo Database
    "odoo_password": "DB Password",                     # The password for your Odoo Database user
    "name": "",                                         # Name of contact person (Optional)
    "phone": ""                                         # Phone number of contact person (Optional)
}

# Send data to the function
response = res_partner_model_instance.get_contact_data(data)

#Get Response
return (response)







# -------------------------- Example code to create a Sale Order  --------------------------

# Create instances of the libraries
sale_order_model_instance = SaleOrderModel()

# Run the Instance
sale_order_model_instance.run()

# Prepare Data
data = {
    "odoo_server_url": "https://exampledb.odoo.com/",   # Your Odoo server URL here (with http or https)
    "database_name": "exampledb",                       # The database name on your Odoo Server 
    "odoo_username": "DB Username",                     # The username for your Odoo Database
    "odoo_password": "DB Password",                     # The password for your Odoo Database user
    "name": "",                                         # Leave it blank as we will generate automatically
    "customerNumber": "",                               # Customer's phone number / customer id in your system
    "customerId": "",                                   # Customer Id from Contacts API
    "resellerNumber": "",                               # Reseller Number (Only required if you are creating an OpenERP account)
    "resellerId": "",                                   # Reseller Id from Accounts API
    "gst_treatment": "",                                # GST Treatment if any ("registered" or "unregister
    "expirationDate": "",                               # Expiry Date in YYYY-MM-DD format
    "quotationDate": "",                                # Quotation Date in format YYYY-MM-DD
    "pricelist": "",                                    # Price List Name
    "orderLine_productNames": ["",""],                  # Product Names for which you want to raise order lines (array of string)
    "orderLine_productId": [],                          # Product Id's for which you want to raise an order (array of numbers)
    "orderLine_description": [],                        # Array of order lines description (array of string)
    "orderLine_quantity":[],                            # Quantity for each product in orderline[] (array of float)
    "orderLine_unitPrice":[],                           # Unit price for each product in orderline[] format (array of float)
    "orderLine_taxes": [],                              # Taxes for each order Line (array of float)
    "orderLine_discount": []                            # Discounts for each order Line (array of float)
}

# Send data to the function
response = sale_order_model_instance.create_sale_order(data)

#Get Response
return (response)




# -------------------------- Example code to get Sales Order Data --------------------------

# Create instances of the libraries
sale_order_model_instance = SaleOrderModel()

# Run the Instance
sale_order_model_instance.run()

# Prepare Data
data = {
    "odoo_server_url": "https://exampledb.odoo.com/",   # Your Odoo server URL here (with http or https)
    "database_name": "exampledb",                       # The database name on your Odoo Server 
    "odoo_username": "DB Username",                     # The username for your Odoo Database
    "odoo_password": "DB Password",                     # The password for your Odoo Database user
    "orderID": "",                                      # The sales order ID you want to retrieve data for
}

# Send data to the function
response = sale_order_model_instance.get_sale_order_data(data)

#Get Response
return (response)





# -------------------------- Example code to Confirm Sales Order Data --------------------------

# Create instances of the libraries
sale_order_model_instance = SaleOrderModel()

# Run the Instance
sale_order_model_instance.run()

# Prepare Data
data = {
    "odoo_server_url": "https://exampledb.odoo.com/",   # Your Odoo server URL here (with http or https)
    "database_name": "exampledb",                       # The database name on your Odoo Server 
    "odoo_username": "DB Username",                     # The username for your Odoo Database
    "odoo_password": "DB Password",                     # The password for your Odoo Database user
    "orderID": "",                                      # The sales order ID you want to Confirm Order
}

# Send data to the function
response = sale_order_model_instance.confirm_sale_order(data)

#Get Response
return (response)
```


Feel free to paste this directly into your README.md file and customize it further if needed.
''',
    url='https://github.com/aditya-infiplus/odooXMLRPCLibrary.git',
    license='MIT',
)